#include "Cscope Control Driver.h"
#include "cscope interface.h"
 

#define	c_init		0
#define	c_acquire	1
#define	c_replay	2
#define	c_check		3
#define	c_update	4
#define	c_finish	5

//Trigger actions:
#define	acq_single	0
#define	acq_auto	1
#define	acq_stop	3

int scope_err;
 
float		a_samples[max_samples];	  	//contains the a channel samples
float		b_samples[max_samples];	  	//contains the b channel samples
unsigned short	dig_samples[max_samples];	//contains teh digotal channel values

unsigned long		samples_required;			//the number of samples to capture and display
unsigned long		samples_returned;			//actual number of samples returned
double		dt,t0;						//holds time increment and start value
unsigned long		frames_returned;			//number of frames returned after sequential transfer
static	   	TD1	acquire;				//holds the acquire definition
static		TD2	error;					//holds the error value
static		LVBoolean	got_samples;	//set to 1 when we have samples
static		long	frame_number;		//used to spec frame number to return when mult-frame replay

//**************************************************************

int call_cscope_control_driver(unsigned short command)
//use this routine to call the control driver with a particular command
{
CscopeControlDriver(command, acquire.StartTime, acquire.StopTime, samples_required,frame_number, &acquire, &got_samples,
					 &t0, &dt, &samples_returned, &frames_returned, a_samples, sizeof(a_samples),  b_samples, sizeof(b_samples),
					 dig_samples, sizeof(dig_samples), &error);

scope_err = error.code;
return scope_err;
}

//****************************************************************
int	scope_init (void)
//Assumes the scope is connected and opens it. 
//Sets up the default values

{
acquire.AcquireMode = 0;		//automatic  (0 = single, 1 = automatic, 2 = triggered)
acquire.AcquisitionMode = 1;	//peak captured
acquire.Acquirer =  4;			//cleverscope is the acquirer
acquire.TransferChans = 2;		//transfer both channels
acquire.AMaxScale = 2;			// Volts range = +/-2
acquire.AMinScale = -2;
acquire.BMaxScale = 2;
acquire.BMinScale = -2;
acquire.AProbe = 0;				//x1
acquire.BProbe = 0;				//x1
acquire.ACoupling = 1;			//DC
acquire.BCoupling = 1;			//DC
acquire.ABandwidth = 1;			//100 MHz
acquire.BBandwidth = 1;			//100 MHz
acquire.TriggerSource = 0;		//A Chan trigger
acquire.TriggerAmplitude = 0;	//Trigger at zero volts
acquire.ATriggerAmplitude = 0;
acquire.BTriggerAmplitude = 0;
acquire.TriggerFilter = 0;		//No trigger filter
acquire.TrigSlope = 0;			//rising 
acquire.TriggerHoldoff = 0;
acquire.DigPatternRqd = 0;		//not used 
acquire.DigPattern =  0;		//not used
acquire.ExtTrigThreshold = 0;
acquire.DigInputThreshold = 2;
acquire.StartTime =  -0.005;	//-5 msecs
acquire.StopTime =   0.005;		//5 msecs
acquire.PreTrigTime = 0.005;
acquire.Port = 0;
acquire.NumDivisions = 10;
acquire.NumSeqFrames = 1; 
acquire.NumBuffers = 2; 
acquire.SigGenFreq = 1000;		//1kHz output
acquire.SigGenAmp = 1;			//1V amplitude
acquire.SigGenOffset = 0;
acquire.SigGenWaveform = 0;		//sine
acquire.SigGenSweep = 0;
acquire.SigGenFunc = 0; 
acquire.SigGenFreq2 = 0;
acquire.SigGenPhase = 0;
acquire.Trig2Function = 0;		//not used
acquire.MinTriggerPeriod = 0.0000001;
acquire.MaxTriggerPeriod = 1; 
acquire.TriggerCount = 1;
acquire.Trig2Slope = 0;
acquire.Trig2SourceChan = 0;
acquire.Trig2Level = 0;
acquire.DigPattern2Rqd = 0;
acquire.DigPattern2 = 0;
acquire.Trigger2Source = 0;
acquire.WaveformAverages = 1;
acquire.ValueChanged = 1;
acquire.SamplerResolution = 0;	//0 = 10 bit (1 = 12 bit, 2 = 14 bit). 
samples_required = 1000;
frame_number =0;
return call_cscope_control_driver(c_init);
}

//**************************************************
int scope_close (void)
//closes the scope
{
return call_cscope_control_driver(c_finish);
}
//****************************************************
void update_values(double a_div, double b_div, double t_div, int number_of_points, 
					double freq, double sigvolts, double trigvolts, int trig_chan, unsigned short trigger_action)
//updates the acquire variable only. If triggered = 1 then we do triggered capture, otherwise auto capture
{
acquire.AMaxScale = v_divisions * a_div / 2;
acquire.AMinScale = - acquire.AMaxScale;
acquire.BMaxScale = v_divisions * b_div / 2;
acquire.BMinScale = - acquire.BMaxScale;
acquire.StopTime = t_divisions * t_div /2;
acquire.StartTime = - acquire.StopTime;  
acquire.SigGenFreq = freq;
acquire.SigGenAmp = sigvolts;
acquire.TriggerAmplitude = trigvolts;
acquire.TriggerSource = trig_chan;	
samples_required = number_of_points; 
acquire.AcquireMode = trigger_action;
acquire.ValueChanged++;
}

//*******************************************************
int scope_config (void)
//Configures major values for the acquisition unit. 
{
return call_cscope_control_driver(c_update);  
}



//*************************************************************************

int	scope_acquire (void)
//start an acquisition

{
return call_cscope_control_driver(c_acquire); 
}


//**************************************************************************
int scope_read_waveform (float a_waveform[max_samples], float b_waveform[max_samples], int *num_samples, double *delta_t, double *t_zero)
//returns the last read waveform for the given channel
{
unsigned int i;

for (i=0; i<samples_returned; i++)
	{
	a_waveform[i] = a_samples[i];
	}
for (i=0; i<samples_returned; i++)
	{
	b_waveform[i] = b_samples[i];
	}

*delta_t = dt;
*t_zero = t0;
*num_samples = samples_returned;
return scope_err;
}

//**************************************************************************

int check_for_samples(void)
//checks to see if samples have been returned. If so returns 1, else 0
{
call_cscope_control_driver(c_check);
return got_samples;

}

//****************************************************************************
