//---------------------------------------------------------------------------
#include <stdlib.h>
#include <stdio.h>
#include <windows.h>
#include <math.h>


#include <vcl.h>

#pragma hdrstop

#include "simple_scope.h"
#include "cscope interface.c"

//**************************************************************************************
//
//
//  See the timer_tick function at the end of this file for where all the action happens
//
//**********************************************************************************

//---------------------------------------------------------------------------
#pragma package(smart_init)
#pragma resource "*.dfm"

#define num_points      1000
#define MAX_PATHNAME_LEN        200

void InitializeGUI (void);

//int 	panel_handle;
double 	a_vert = 1.0, b_vert = 1.0, horz = 0.001;
double	sig_gen_volts = 1.0, sig_gen_freq = 1000.0;
int	channel = 1;
int 	trig_channel = 0;
double	trig_volts = 0.0;
int	single_acquire = false;
int	auto_acquire = false;
float  a_waveform[max_samples], b_waveform[max_samples];
double	dT, T0;
char 	pathname[MAX_PATHNAME_LEN];
int	waiting_for_trigger;	//0 means we have not started an acquire. 1 means we are waiting for the trigger
unsigned short	trigger_action;	//0 = auto, 1 = single, 3 = stop

int	num_samples;
int	vals_changed;

//****************************************************************************
//Support routines

double v_div[9] = { 5,2,1,0.5,0.2,0.1,0.05,0.02,0.01};
double t_div[25] = { 1,0.5,0.2,0.1,0.05,0.02,0.01,0.005,0.002,0.001,
					  500e-6,200e-6,100e-6, 50e-6,20e-6,10e-6,5e-6,2e-6,1e-6,
					  500e-9,200e-9,100e-9,50e-9,20e-9,10e-9};
double freqs[21] = {1.0,2,5,10,20,50,100,200,500,1000,2000,5000,10000,
					 20000,50000,100000,200000,500000,1000000,2000000,5000000};

//-------------------------------------------------------------------------
#define PREFIX_START (-24)
/* Smallest power of then for which there is a prefix defined.
   If the set of prefixes will be extended, change this constant
   and update the table "prefix". */

char *eng(double value, int digits, int numeric)
{
  static char *prefix[] = {
  "y", "z", "a", "f", "p", "n", "u", "m", "",
  "k", "M", "G", "T", "P", "E", "Z", "Y"
  };
#define PREFIX_END (PREFIX_START+\
(int)((sizeof(prefix)/sizeof(char *)-1)*3))

      int expof10;
      static unsigned char result[100];
      unsigned char *res = result;

      if (value < 0.)
        {
            *res++ = '-';
            value = -value;
        }
		if (value != 0.0)
			{
			expof10 = (int) log10(value);
		   if(expof10 > 0)
			 expof10 = (expof10/3)*3;
		   else
			 expof10 = (-expof10+3)/3*(-3);

		   value *= pow(10,-expof10);

		   if (value >= 1000.)
			  { value /= 1000.0; expof10 += 3; }
		   else if(value >= 100.0)
			  digits -= 2;
		   else if(value >= 10.0)
			  digits -= 1;
		   }
	   else
			{
			expof10 = 0;	//no prefix
			digits=3;
			}
      if(numeric || (expof10 < PREFIX_START) ||
                    (expof10 > PREFIX_END))
        sprintf(res, "%.*fe%d", digits-1, value, expof10);
      else
        sprintf(res, "%.*f %s", digits-1, value,
          prefix[(expof10-PREFIX_START)/3]);
      return result;
}
//------------------------------------------------------------------------
void Setval(TLabel* ctrl, double value, char *suffix)
{
	  CHAR ctrl_val[10];
	  sprintf(ctrl_val, "%s%s", eng(value, 3, 0),suffix);
	  ctrl->Caption = ctrl_val;
}

//***************************************************************************
TForm1 *Form1;
//---------------------------------------------------------------------------
__fastcall TForm1::TForm1(TComponent* Owner)
        : TForm(Owner)
{
}
//---------------------------------------------------------------------------


void InitializeGUI ()
{
single_acquire = false;
auto_acquire = false;
waiting_for_trigger = false;
vals_changed = true;
trigger_action = acq_stop;   //stop triggering
TeeDefaultCapacity = 8000;     //allocate enough room for 8000 samples
Setval(Form1->Adiv, a_vert,"V");
Setval(Form1->Bdiv, b_vert,"V");
Setval(Form1->Tdiv, horz,"s");
Setval(Form1->TrigV, trig_volts,"V");
Setval(Form1->SigGenF, sig_gen_freq,"Hz");
Setval(Form1->SigGenV, sig_gen_volts,"V");
Form1->SetA_Vertical-> Position = 2;
Form1->Chart1->LeftAxis->Minimum = -a_vert*v_divisions/2;
Form1->Chart1->LeftAxis->Maximum = a_vert*v_divisions/2 ;
Form1->Chart1->LeftAxis->Increment = a_vert;
Form1->SetB_Vertical-> Position = 2;
Form1->Chart1->RightAxis->Minimum = -a_vert*v_divisions/2;
Form1->Chart1->RightAxis->Maximum = a_vert*v_divisions/2 ;
Form1->Chart1->RightAxis->Increment = b_vert;
Form1->SetHorizontal-> Position = 9;
Form1->Chart1->BottomAxis->Minimum = -horz*t_divisions/2;
Form1->Chart1->BottomAxis->Maximum = horz*t_divisions/2 ;
Form1->Chart1->BottomAxis->Increment = horz;
Form1->Timer1 ->Enabled = true;
Form1->Trig_A->Checked = true;
}
//**************************************************************************
void __fastcall TForm1::FormClose(TObject *Sender, TCloseAction &Action)
{
scope_close ();
}
//---------------------------------------------------------------------------

void __fastcall TForm1::QuitButtonClick(TObject *Sender)
{
scope_close ();
Sleep(500);		//allow run time system to close sown
Close();
}


//---------------------------------------------------------------------------
void __fastcall TForm1::SetA_VerticalChange(TObject *Sender)
{
a_vert =  v_div[SetA_Vertical-> Position];
Setval(Form1->Adiv, a_vert,"V");
vals_changed = true;
Chart1->LeftAxis->Minimum = -a_vert*v_divisions/2;
Chart1->LeftAxis->Maximum = a_vert*v_divisions/2 ;
Chart1->LeftAxis->Increment = a_vert;

}
//---------------------------------------------------------------------------

void __fastcall TForm1::SetHorizontalChange(TObject *Sender)
{
horz =  t_div[SetHorizontal-> Position];
Setval(Form1->Tdiv, horz,"s");
vals_changed = true;
Chart1->BottomAxis->Minimum = -horz*t_divisions/2;
Chart1->BottomAxis->Maximum = horz*t_divisions/2 ;
Chart1->BottomAxis->Increment = horz;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::SetB_VerticalChange(TObject *Sender)
{
b_vert =  v_div[SetB_Vertical-> Position];
Setval(Form1->Bdiv, b_vert,"V");
vals_changed = true;
Chart1->RightAxis->Minimum = -b_vert*v_divisions/2;
Chart1->RightAxis->Maximum = b_vert*v_divisions/2 ;
Chart1->RightAxis->Increment = b_vert;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Set_SigGen_FreqChange(TObject *Sender)
{
sig_gen_freq =  freqs[Set_SigGen_Freq-> Position];
Setval(Form1->SigGenF, sig_gen_freq, "Hz");
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Set_SigGen_VoltsChange(TObject *Sender)
{
sig_gen_volts = Set_SigGen_Volts-> Position * 0.2;
Setval(Form1->SigGenV, sig_gen_volts, "V");
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Set_Trigger_VoltsChange(TObject *Sender)
{
trig_volts = Set_Trigger_Volts-> Position * -0.2;
Setval(Form1->TrigV, trig_volts, "V");
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Trig_AClick(TObject *Sender)
{
trig_channel = 0;
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::Trig_BClick(TObject *Sender)
{
trig_channel = 1;
vals_changed = true;
}
//---------------------------------------------------------------------------

void __fastcall TForm1::FormActivate(TObject *Sender)
{
int		nErr;
char	message[100];

	 nErr =scope_init ();
	 if (nErr>0)
		{
		 MessageDlg("There was a problem with the Cleverscope DLL, the error code = " + IntToStr(nErr),
				mtInformation, TMsgDlgButtons() << mbOK, 0);

		}


	 InitializeGUI();

}
//---------------------------------------------------------------------------


void __fastcall TForm1::SingleAcquireClick(TObject *Sender)
{
vals_changed = true;
Trig_LED -> Brush -> Color = clWhite;
auto_acquire = false;
waiting_for_trigger = false;
TriggeredAcquire->Caption="Auto";

if (single_acquire)
	{
	single_acquire = false;
	SingleAcquire->Caption="Single";
	trigger_action = acq_stop;   //stop triggering
	}
else
	{
	single_acquire= true;
	SingleAcquire->Caption="Wait...";
	trigger_action = acq_single;	//use triggered capture
	}
}
//---------------------------------------------------------------------------


void __fastcall TForm1::TriggeredAcquireClick(TObject *Sender)
{
vals_changed = true;
single_acquire = false;
Trig_LED -> Brush -> Color = clWhite;
waiting_for_trigger = false;
SingleAcquire->Caption="Single";
if (auto_acquire)
	{
	auto_acquire = false;
	TriggeredAcquire->Caption="Auto";
	trigger_action = acq_stop;   //stop triggering
	}
else
	{
	auto_acquire = true;
	TriggeredAcquire->Caption="Triggering";
	trigger_action = acq_auto;	//use auto capture
	}
}
//---------------------------------------------------------------------------
//This is the main event function. It checks to see if any values have changed, and if
//they have, it updates the acquire variable.
//Next it sees if an acquisition is to be made - and if so, makes it.
//Next it checks if there are results - if so, get them
//Finally it checks if the changes that should have been updated by teh acquire (if it happened)
//have happened, and if not, it updates the acquisition unit itself.


void __fastcall TForm1::Timer1Timer(TObject *Sender)
{

int	i;
double	xval;

if (vals_changed)
    {
	 update_values(a_vert, b_vert, horz, num_points,
	 sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action);
	 }
if ((!waiting_for_trigger) && (single_acquire || auto_acquire))
    {
	 Trig_LED -> Brush -> Color = clWhite;
	 waiting_for_trigger = true;
    scope_acquire ();
	 vals_changed = false;		//will have been updated by acquire
	 }
if ((waiting_for_trigger) && check_for_samples())
	 {
	  Trig_LED -> Brush -> Color = clRed;
	  waiting_for_trigger = false;
	  if (single_acquire)  SingleAcquire->Caption="Single";
	  single_acquire = false;
	  scope_read_waveform (a_waveform, b_waveform, &num_samples, &dT, &T0);
	  xval = T0 ;
	  Series1->Clear();
	  Series2->Clear();
	  for (i=0; i < num_samples; i++)
		  {
		  Series1->AddXY(xval,a_waveform[i]);
		  Series2->AddXY(xval,b_waveform[i]);
		  xval = xval+dT;
		  }
	  }
if (vals_changed == true)
	  {
	  update_values(a_vert, b_vert, horz, num_points,
	  sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action);
	  scope_config();
	  vals_changed = false;		//updated if acquire did not update them
	  }

}
//---------------------------------------------------------------------------


