#include <cvirte.h>    /* Needed if linking in external compiler; harmless otherwise */
#include <userint.h>
#include <utility.h>
#include <ansi_c.h>
#include "simplescope.h"
#include "cscope interface.h"

#define	num_points	1000


void InitializeGUI (void);

int 	panel_handle;
double 	a_vert = 0.5, b_vert = 0.5, horz = 0.0002;
double	sig_gen_volts = 1.0, sig_gen_freq = 1000.0;
int		channel = 1;
int 	trig_channel = 0;
double	trig_volts = 0.0;
int		single_acquire = 0;
int		auto_acquire = 0;
float 	a_waveform[max_samples], b_waveform[max_samples];
double	x_delta, xzero;
char 	pathname[MAX_PATHNAME_LEN];
int		waiting_for_trigger = 0;	//0 means we have not started an acquire. 1 means we are waiting for the trigger
int		num_samples;
int		vals_changed = 0;
unsigned short	trigger_action = 3;	//0 = auto, 1 = single, 3 = stop

//---------------------------------------------------------------------------

int main (int argc, char *argv[])
{
int		nErr;
char	message[100];

    if (InitCVIRTE (0, argv, 0) == 0)    /* Needed if linking in external compiler; harmless otherwise */
        return -1;    /* out of memory */

    nErr =scope_init ();
    if (nErr>0)
    	{
    	sprintf (message, "There was a problem with the Cleverscope DLL, the error code = %d", nErr);
    	MessagePopup ("Problem with Cleverscope DLL", message);
    	return -1;
    	}

    panel_handle = LoadPanel (0, "SimpleScope.uir", PANEL);
    waiting_for_trigger = 0;
    vals_changed = 1;
    DisplayPanel (panel_handle);
    InitializeGUI();
    nErr = RunUserInterface ();
	scope_close ();
    CloseCVIRTE();
    return 0;
}
//---------------------------------------------------------------------------

int  CVICALLBACK SingleAcquire(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
    	{
        GetCtrlVal (panel_handle, PANEL_SINGLE, &single_acquire);
        vals_changed = 1;
        auto_acquire = 0;
        waiting_for_trigger = 0;
        trigger_action = 1;
   		SetCtrlVal (panel_handle, PANEL_TRIGGERED, auto_acquire);
        SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
		if (single_acquire)
			{
			trigger_action = acq_single;   //use triggered capture
			}
		else
			{
			trigger_action = acq_stop;	//stop triggering
			}

    	}
    return(0);

}
//---------------------------------------------------------------------------

int  CVICALLBACK TriggeredAcquire(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT)
    	{
        GetCtrlVal (panel_handle, PANEL_TRIGGERED, &auto_acquire);
        vals_changed = 1;
		single_acquire = 0;
        waiting_for_trigger = 0;
   		SetCtrlVal (panel_handle, PANEL_SINGLE, single_acquire);
        SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
        if (auto_acquire)
			{
			trigger_action = acq_auto;   //stop triggering
			}
		else
			{
			trigger_action = acq_stop;	//use auto capture
			}
        }
    return(0);

}
//---------------------------------------------------------------------------

int CVICALLBACK UpdateCallback (int panel, int control, int event,
        void *callbackData, int eventData1, int eventData2)
{
    switch (event)
    {
        case EVENT_TIMER_TICK:
               if (vals_changed==1)
            	{
            	update_values(a_vert, b_vert, horz, num_points,
				sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action);
            	}
            if ((waiting_for_trigger == 0) && (single_acquire || auto_acquire))
            	{
            	SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
            	scope_acquire ();
            	waiting_for_trigger = 1;
            	vals_changed = 0;		//will have been updated by acquire
            	}
           	if ((waiting_for_trigger == 1) && check_for_samples())
       			{
       			SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 1);
				single_acquire = 0;
				SetCtrlVal (panel_handle, PANEL_SINGLE, single_acquire);
       			waiting_for_trigger = 0;
       			scope_read_waveform (a_waveform, b_waveform, &num_samples, &x_delta, &xzero);
        		DeleteGraphPlot (panel_handle, PANEL_GRAPH, -1, 0);
				SetCtrlAttribute (panel_handle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS,VAL_LEFT_YAXIS);
				PlotWaveform (panel_handle, PANEL_GRAPH, a_waveform, num_samples,
					VAL_FLOAT, 1, 0, xzero, x_delta,
					VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_RED);
				SetCtrlAttribute (panel_handle, PANEL_GRAPH, ATTR_ACTIVE_YAXIS,VAL_RIGHT_YAXIS);
				PlotWaveform (panel_handle, PANEL_GRAPH, b_waveform, num_samples,
					VAL_FLOAT, 1, 0, xzero, x_delta,
					VAL_THIN_LINE, VAL_EMPTY_SQUARE, VAL_SOLID, 1, VAL_BLUE);
           		}
           	if (vals_changed == 1)
           		{
             	update_values(a_vert, b_vert, horz, num_points,
				sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action);
           		scope_config();
           		vals_changed = 0;		//updated if acquire did not update them
           		}
            break;
    }

    return 0;
}

//---------------------------------------------------------------------------

int  CVICALLBACK SetA_Vertical(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_A_VERTICAL, &a_vert);
        SetCtrlVal (panel_handle, PANEL_VA_READOUT, a_vert);
		SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL, -a_vert*v_divisions/2, a_vert*v_divisions/2);
        vals_changed = 1;
    }
    return (0);
}
//---------------------------------------------------------------------------

int  CVICALLBACK SetB_Vertical(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_B_VERTICAL, &b_vert);
        SetCtrlVal (panel_handle, PANEL_VB_READOUT, b_vert);
		SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_RIGHT_YAXIS, VAL_MANUAL, -b_vert*v_divisions/2, b_vert*v_divisions/2);

        vals_changed = 1;
    }
    return (0);
}

//---------------------------------------------------------------------------

int CVICALLBACK SetHorizontal(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_HORIZONTAL, &horz);
        SetCtrlVal (panel_handle, PANEL_HREADOUT, horz);
		SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_BOTTOM_XAXIS, VAL_MANUAL, -horz*t_divisions/2, horz*t_divisions/2);
        vals_changed = 1;
    }
    return(0);
}
//---------------------------------------------------------------------------

int CVICALLBACK SetChannel(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_COMMIT) {
        GetCtrlVal (panel_handle, PANEL_CHANNEL, &trig_channel);
        vals_changed = 1;
    }
    return(0);

}
//---------------------------------------------------------------------------

int  CVICALLBACK Set_SigGen_Volts(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_SIGGENVOLTS, &sig_gen_volts);
        SetCtrlVal (panel_handle, PANEL_SIGGENVREADOUT, sig_gen_volts);
        vals_changed = 1;
    }
    return (0);
}
//---------------------------------------------------------------------------

int  CVICALLBACK SetSigGenFreq(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_SIGGENFREQ, &sig_gen_freq);
        SetCtrlVal (panel_handle, PANEL_SIGGENFREADOUT, sig_gen_freq);
        vals_changed = 1;
    }
    return (0);
}
//---------------------------------------------------------------------------

int  CVICALLBACK Set_Trigger_Volts(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    if (event == EVENT_VAL_CHANGED) {
        GetCtrlVal (panel_handle, PANEL_TRIGGER, &trig_volts);
        SetCtrlVal (panel_handle, PANEL_TRIGGERREADOUT, trig_volts);
        vals_changed = 1;
    }
    return (0);
}

//---------------------------------------------------------------------------

int CVICALLBACK QuitButton(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{


    if (event == EVENT_COMMIT) {

        SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);
    	SetCtrlAttribute (PANEL, PANEL_TIMER, ATTR_ENABLED, 0 );  //turn off the event timer
        QuitUserInterface (0);
        }
    return(0);
}
//---------------------------------------------------------------------------

int CVICALLBACK SaveToDisk(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char directory[MAX_PATHNAME_LEN];
    FILE *file_handle;
    int i;

    if (event == EVENT_COMMIT) {
        GetProjectDir (directory);
        if (FileSelectPopup (directory, "*.wfm", "WFM Files",
                             "Save Waveform to Disk", VAL_SAVE_BUTTON, 1,
                             1, 1, 0, pathname) > 0) {
        file_handle = fopen (pathname, "w+");
        fprintf (file_handle, "OSCILLOSCOPE DEMO DATA FILE \n\n");
        fprintf(file_handle, "Channel Acquired: CH%d\n", channel);
        fprintf(file_handle, "Date %s   Time: %s \n", DateStr(), TimeStr());
        fprintf (file_handle, "----------------------------------\n");
        for (i=0;i<100;i++)
            fprintf (file_handle, "%f\n", a_waveform[i]);
        fclose (file_handle);
        SetCtrlAttribute (panel_handle, PANEL_VIEWDATA, ATTR_DIMMED, 0);
        }
    }
    return (0);
}

//---------------------------------------------------------------------------

int CVICALLBACK ViewDataPoints(int panel, int control, int event, void *callbackData, int eventData1, int eventData2)
{
    char filestring[100];

    if (event == EVENT_COMMIT) {
        sprintf (filestring, "notepad.exe %s", pathname);
        LaunchExecutable (filestring);
        }
    return (0);
}
//---------------------------------------------------------------------------

void InitializeGUI ()
{
	single_acquire = 0;
	auto_acquire = 0;
	waiting_for_trigger = 0;
	vals_changed = 1;
	trigger_action = acq_stop;   //stop triggering
    SetCtrlVal (panel_handle, PANEL_VA_READOUT, a_vert);
    SetCtrlVal (panel_handle, PANEL_VB_READOUT, b_vert);
    SetCtrlVal (panel_handle, PANEL_HREADOUT, horz);
    SetCtrlVal (panel_handle, PANEL_TRIGGERREADOUT, trig_volts);
    SetCtrlVal (panel_handle, PANEL_SIGGENVREADOUT, sig_gen_volts);
    SetCtrlVal (panel_handle, PANEL_SIGGENFREADOUT, sig_gen_freq);
    SetCtrlVal (panel_handle, PANEL_CHANNEL, trig_channel);
    SetCtrlVal (panel_handle, PANEL_TRIGGER_LED, 0);

    SetCtrlVal (panel_handle, PANEL_A_VERTICAL, a_vert);
    SetCtrlVal (panel_handle, PANEL_B_VERTICAL, b_vert);
    SetCtrlVal (panel_handle, PANEL_HORIZONTAL, horz);
    SetCtrlVal (panel_handle, PANEL_TRIGGER, trig_volts);
    SetCtrlVal (panel_handle, PANEL_SIGGENVOLTS, sig_gen_volts);
    SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_BOTTOM_XAXIS, VAL_MANUAL, -horz*t_divisions/2, horz*t_divisions/2);
	SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_LEFT_YAXIS, VAL_MANUAL, -a_vert*v_divisions/2, a_vert*v_divisions/2);
	SetAxisScalingMode (panel_handle, PANEL_GRAPH, VAL_RIGHT_YAXIS, VAL_MANUAL, -b_vert*v_divisions/2, b_vert*v_divisions/2);
}
//---------------------------------------------------------------------------
