unit cscope_delphi_unit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Math,
  Dialogs, StdCtrls, TeEngine, Series, ExtCtrls,
  TeeProcs, Chart, ComCtrls, cscope_interface;

type
  TForm1 = class(TForm)
    Chart1: TChart;
    Timer1: TTimer;
    Series1: TFastLineSeries;
    Series2: TFastLineSeries;
    Set_SigGen_Freq: TTrackBar;
    Set_SigGen_Volts: TTrackBar;
    SetHorizontal: TTrackBar;
    SetA_Vertical: TTrackBar;
    SetB_Vertical: TTrackBar;
    Set_Trigger_Volts: TTrackBar;
    Label1: TLabel;
    Label2: TLabel;
    SigGenF: TLabel;
    SigGenV: TLabel;
    Label5: TLabel;
    TrigV: TLabel;
    Label7: TLabel;
    Tdiv: TLabel;
    Adiv: TLabel;
    Bdiv: TLabel;
    Trig_A: TRadioButton;
    Trig_B: TRadioButton;
    Trig_LED: TShape;
    SingleAcquire: TButton;
    TriggeredAcquire: TButton;
    QuitButton: TButton;
    Label12: TLabel;
    Label13: TLabel;
    Bevel1: TBevel;
    Bevel2: TBevel;
    Bevel3: TBevel;
    Label3: TLabel;
    Label4: TLabel;
    Label6: TLabel;
    Label8: TLabel;
    Label9: TLabel;
    Label10: TLabel;
    Label11: TLabel;
    Label14: TLabel;
    Label15: TLabel;
    Timer_val: TLabel;

    procedure Timer1Timer(Sender: TObject);
    procedure FormCreate(Sender: TObject);
    procedure QuitButtonClick(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);
    procedure TriggeredAcquireClick(Sender: TObject);
    procedure SingleAcquireClick(Sender: TObject);
    procedure SetA_VerticalChange(Sender: TObject);
    procedure SetB_VerticalChange(Sender: TObject);
    procedure SetHorizontalChange(Sender: TObject);
    procedure Set_Trigger_VoltsChange(Sender: TObject);
    procedure Trig_AClick(Sender: TObject);
    procedure Trig_BClick(Sender: TObject);
    procedure Set_SigGen_FreqChange(Sender: TObject);
    procedure Set_SigGen_VoltsChange(Sender: TObject);

  private
    { Private declarations }
  public
    { Public declarations }
	end;



var
  Form1: TForm1;

implementation

{$R *.dfm}

var
a_vert,b_vert,horz: double;
sig_gen_volts, sig_gen_freq: double;
channel, trig_channel : integer;
trig_volts: double;
single_acquire, auto_acquire : boolean;
trigger_action: word;		//How to trigger, 0 = single, 1 = auto, 3 = stop
a_waveform, b_waveform: signal;
dT, T0: double;
waiting_for_trigger: boolean;
num_samples, rqd_samples: integer;
vals_changed: boolean;
last_time, this_time:integer;

//****************************************************************************
//Support routines

const
v_div:array[0..8] of double = (5,2,1,0.5,0.2,0.1,0.05,0.02,0.01);
t_div:array[0..24] of double = ( 1,0.5,0.2,0.1,0.05,0.02,0.01,0.005,0.002,0.001,
					  500e-6,200e-6,100e-6, 50e-6,20e-6,10e-6,5e-6,2e-6,1e-6,
					  500e-9,200e-9,100e-9,50e-9,20e-9,10e-9);
freqs:array[0..20] of double = (1.0,2,5,10,20,50,100,200,500,1000,2000,5000,10000,
					 20000,50000,100000,200000,500000,1000000,2000000,5000000);
prefix:string = 'yzafpnum kMGTPEZY';
prefix_small = -24;
prefix_large = 24;

function eng(value:double; digits:integer; numeric_exponent:boolean):string;
{formats a number as an engineering string}

var  expof10:integer;
     eng_str:string;
     neg:boolean;

begin
neg:=false;
if value < 0.0 then
  begin
  neg:=true;
  value := -value;
  end;
if value <> 0.0 then
	begin
   expof10 := round(log10(value));
   if expof10 > 0 then
      expof10 := (expof10 div 3)*3
	else
   	expof10 := ((-expof10+3) div 3)*(-3);

 	value := value * power(10,-expof10);

   if (value >= 1000.0) then
   	begin
       value:= value / 1000.0;
       expof10 := expof10 +3;
       end
    else
    	begin
       if value >= 100.0 then
       	begin
       	digits := digits - 2;
           end
    	else
       	begin
           if value >= 10.0 then
           	begin
       		digits := digits - 1;
               end;
           end;
       end;
	end
else
	begin
   expof10 := 0;	//no prefix
   digits:=3;
   end;
if(numeric_exponent or (expof10 < prefix_small) or (expof10 > prefix_large)) then
  eng_str := format('%.*fe%d',[digits-1, value, expof10])
else
  eng_str:= format('%.*f %s',[digits-1, value,prefix[(expof10-prefix_small+3) div 3]]);
if neg then eng_str:='-'+eng_str;
eng:=eng_str;
end;

//------------------------------------------------------------------------
procedure Setval(var ctrl: TLabel; value: double; suffix: string);
{stores the number to the caption}

begin
ctrl.Caption := eng(value, 3, false) + suffix;
end;

//***************************************************************************
procedure InitializeGUI;

begin
with Form1 do
	begin
   last_time:=0;
  a_vert := 1;
  b_vert := 1;
  horz := 0.001;
  sig_gen_volts := 1.0;
  sig_gen_freq := 1000.0;
  rqd_samples:=1000;
  channel := 1;
  trig_channel := 0;
  trig_volts := 0.0;
  single_acquire := false;
  auto_acquire := false;
  trigger_action:=acq_stop;   //no capture
  waiting_for_trigger:=false;
  vals_changed := true;
  TeeDefaultCapacity := 8000;     //allocate enough room for 8000 samples
  Setval(Adiv, a_vert,'V');
  Setval(Bdiv, b_vert,'V');
  Setval(Tdiv, horz,'s');
  Setval(TrigV, trig_volts,'V');
  Setval(SigGenF, sig_gen_freq,'Hz');
  Setval(SigGenV, sig_gen_volts,'V');
  SetA_Vertical.Position := 2;
  Chart1.LeftAxis.Minimum := -a_vert*v_divisions/2;
  Chart1.LeftAxis.Maximum := a_vert*v_divisions/2 ;
  Chart1.LeftAxis.Increment := a_vert;
  SetB_Vertical. Position := 2;
  Chart1.RightAxis.Minimum := -a_vert*v_divisions/2;
  Chart1.RightAxis.Maximum := a_vert*v_divisions/2 ;
  Chart1.RightAxis.Increment := b_vert;
  SetHorizontal. Position := 9;
  Chart1.BottomAxis.Minimum := -horz*t_divisions/2;
  Chart1.BottomAxis.Maximum := horz*t_divisions/2 ;
  Chart1.BottomAxis.Increment := horz;
  Trig_A.Checked := true;
  Timer1.Enabled := true;
  end;
end;
//**************************************************************************


procedure TForm1.FormCreate(Sender: TObject);

var	error:integer;

begin
scope_init (error);

if (error>0) then
	MessageDlg('There was a problem with the Cleverscope DLL, the error code = ' + IntToStr(error),
				mtInformation, [mbOK], 0);

InitializeGUI();
end;
//**************************************************************************
procedure TForm1.QuitButtonClick(Sender: TObject);
var	error:integer;

begin
timer1.Enabled:=false;
scope_close(error);
sleep(500);    //time to for the runtime system to close down
close;
end;
//**************************************************************************
procedure TForm1.FormClose(Sender: TObject; var Action: TCloseAction);
var	error:integer;

begin
timer1.Enabled:=false;
scope_close(error);
sleep(500); //time to for the runtime system to close down
end;
//**************************************************************************
procedure TForm1.SetA_VerticalChange(Sender: TObject);
begin
a_vert :=  v_div[SetA_Vertical.Position];
Setval(Form1.Adiv, a_vert,'V');
vals_changed := true;
Chart1.LeftAxis.Minimum := -a_vert*v_divisions/2;
Chart1.LeftAxis.Maximum := a_vert*v_divisions/2 ;
Chart1.LeftAxis.Increment := a_vert;
end;
//**************************************************************************
procedure TForm1.SetB_VerticalChange(Sender: TObject);
begin
b_vert :=  v_div[Setb_Vertical.Position];
Setval(Form1.Bdiv, b_vert,'V');
vals_changed := true;
Chart1.RightAxis.Minimum := -b_vert*v_divisions/2;
Chart1.RightAxis.Maximum := b_vert*v_divisions/2 ;
Chart1.RightAxis.Increment := b_vert;
end;
//**************************************************************************
procedure TForm1.SetHorizontalChange(Sender: TObject);
begin
horz :=  t_div[SetHorizontal. Position];
Setval(Form1.Tdiv, horz,'s');
vals_changed := true;
Chart1.BottomAxis.Minimum := -horz*t_divisions/2;
Chart1.BottomAxis.Maximum := horz*t_divisions/2 ;
Chart1.BottomAxis.Increment := horz;

end;
//**************************************************************************
procedure TForm1.Set_Trigger_VoltsChange(Sender: TObject);
begin
trig_volts := Set_Trigger_Volts. Position * -0.2;
Setval(Form1.TrigV, trig_volts, 'V');
vals_changed := true;

end;
//**************************************************************************
procedure TForm1.Trig_AClick(Sender: TObject);
begin
trig_channel := 0;
vals_changed := true;
end;
//**************************************************************************
procedure TForm1.Trig_BClick(Sender: TObject);
begin
trig_channel := 1;
vals_changed := true;
end;
//**************************************************************************
procedure TForm1.Set_SigGen_FreqChange(Sender: TObject);
begin
sig_gen_freq :=  freqs[Set_SigGen_Freq. Position];
Setval(Form1.SigGenF, sig_gen_freq, 'Hz');
vals_changed := true;

end;
//**************************************************************************
procedure TForm1.Set_SigGen_VoltsChange(Sender: TObject);
begin
sig_gen_volts := Set_SigGen_Volts. Position * 0.2;
Setval(Form1.SigGenV, sig_gen_volts, 'V');
vals_changed := true;

end;
//**************************************************************************

procedure TForm1.SingleAcquireClick(Sender: TObject);
begin
vals_changed := true;
Trig_LED.Brush.Color := clWhite;
auto_acquire := false;
waiting_for_trigger := false;
TriggeredAcquire.Caption:='Auto';
if (single_acquire) then
	begin
   single_acquire := false;
   SingleAcquire.Caption:='Single';
   trigger_action :=acq_stop;
   end
else
	begin
   single_acquire := true;
   SingleAcquire.Caption:='Wait...';
   trigger_action :=acq_single;
   end

end;
//**************************************************************************

procedure TForm1.TriggeredAcquireClick(Sender: TObject);
begin
vals_changed := true;
single_acquire := false;
Trig_LED.Brush.Color:=clWhite;
waiting_for_trigger :=false;
SingleAcquire.Caption:='Single';
if (auto_acquire) then
	begin
   trigger_action:=acq_stop;
	auto_acquire := false;
	TriggeredAcquire.Caption:='Auto';
	end
else
	begin
	auto_acquire := true;
	TriggeredAcquire.Caption:='Triggering...';
   trigger_action:=acq_auto;
	end;
end;

 //**************************************************************************

procedure TForm1.Timer1Timer(Sender: TObject);
var     i: integer;
        xval: double;
        error :integer;
        delta_time:integer;

begin

if vals_changed then
    begin
    update_values(a_vert, b_vert, horz, rqd_samples,
	 sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action);
    {update values here so that if we do an acquire we use the new values}
	 end;
if ((not waiting_for_trigger) and (single_acquire or auto_acquire)) then
    begin
    {start the next acquisition}
    Trig_LED.Brush.Color := clWhite;
    waiting_for_trigger := true;
    vals_changed := false;
    scope_acquire (error);
    end;
    
if (waiting_for_trigger and check_for_samples) then
	{If samples ready, we get them}
     begin
     {
		this_time:=(DateTimeToTimeStamp(now)).time;
		delta_time:=this_time-last_time;
		last_time:=this_time;
     	Timer_val.Caption:=IntToStr(delta_time);
       }
     Trig_LED.Brush.Color := clRed;
     waiting_for_trigger:=false;
     if (single_acquire) then  SingleAcquire.Caption:='Single';
     single_acquire := false;
     scope_read_waveform (a_waveform, b_waveform, num_samples,dT,T0,error);
     xval := T0 ;
     Series1.Clear;
     Series2.Clear;
     for i:=0 to num_samples-1 do
         begin
         Series1.AddXY(xval,a_waveform[i]);
         Series2.AddXY(xval,b_waveform[i]);
         xval := xval+dT;
         end;
     end;

if vals_changed then
    begin
    update_values(a_vert, b_vert, horz, rqd_samples,
	 sig_gen_freq,sig_gen_volts,trig_volts, trig_channel,trigger_action);
    {update values here acquire did not update them}
    scope_config (error);
    vals_changed := false;
	 end;
end;
//**************************************************************************

end.
