unit cscope_interface;                                                       

interface

uses     Cscope_control_driver;

const    max_samples =  16384;
			t_divisions=10;
			v_divisions=8;
           acq_single=0;
			acq_auto=1;
			acq_stop=3;

           
type    signal = array[0..max_samples-1] of single;

procedure scope_init(var error_code:integer);
         {initializes cleverscope dll. Returns 0 if all ok, otherwise error code}
procedure scope_close(var error_code:integer);
         {closes cleverscope dll. Returns 0 if all ok, otherwise error code}

procedure scope_config (var error_code:integer);
         {configures the acquisition unit, using the current acquire value}


procedure	scope_acquire (var error_code:integer);
           {starts an acquisition. returns error code, = 0 if ok}

procedure scope_read_waveform (var a_waveform, b_waveform : signal;
                 var num_samples:integer; var delta_t, t_zero : double;
                 var error_code:integer);
          {reads the samples from the buffer, including waveform details}

function check_for_samples:boolean;
         {returns true if we have got the samples}

procedure update_values(a_div, b_div, t_div :double ; number_of_points:integer;
			freq,sigvolts,trigvolts:double; trig_chan:integer; trigger_action:word);
           {updates the values in the acquire structure but does not send them
            to the scope - you need to run either Scope_acquire or scope_config
            to do that. trigger_action = single (0), auto (1), or stop (3)
            Trig_chan = 0 for Chan A, 1 = chan B}

//------------------------------------------------------------------
implementation

type        commands =  (c_init,c_acquire,c_replay, c_check, c_update, c_finish);

var
a_samples, b_samples:	array[0..max_samples-1] of single;   //contains the a, b channel samples
dig_samples:			array[0..max_samples-1] of word;	//contains teh digotal channel values
samples_required:		integer;			//the number of samples to capture and display
samples_returned:		integer;			//actual number of samples returned
dt,t0:					double;				//holds time increment and start value
frames_returned:		integer;			//number of frames returned after sequential transfer
acquire: 				Acquire_record;	   //holds the acquire definition
errorOut: 				Error_record;				//holds the error value
got_samples: 			byte;               //set to 1 when we have samples
frame_number:			integer;			//the frame to get, if a multi-frame return done.

//**************************************************************
procedure call_control_driver(command: commands);
//works totally with local globals to make it easy

begin
CscopeControlDriver(word(command),acquire.StartTime,acquire.StopTime,
	samples_required,frame_number,@Acquire,@got_samples,@t0,@dT,
   @samples_returned,@frames_returned,
   @a_samples[0],sizeof(a_samples),@b_samples[0],sizeof(b_samples),
	@dig_samples[0],sizeof(dig_samples), @errorOut) ;
end;

//**************************************************************
procedure scope_init(var error_code:integer);
//Assumes the scope is connected and opens it. 
//Sets up the default values

begin
acquire.AcquireMode := 1;    //0 single,  1 automatic
acquire.AcquisitionMode := 1;    //peak captured
acquire.Acquirer :=  4;    //cleverscope is the acquirer
acquire.TransferChans := 2;    //transfer both channels
acquire.AMaxScale := 5;    // Volts range := +/-2
acquire.AMinScale := -5;
acquire.BMaxScale := 5;
acquire.BMinScale := -5;
acquire.AProbe := 0;    //x1
acquire.BProbe := 0;    //x1
acquire.ACoupling := 1;    //DC
acquire.BCoupling := 1;    //DC
acquire.ABandwidth := 1;    //100 MHz
acquire.BBandwidth := 1;    //100 MHz
acquire.TriggerSource := 0;    //A Chan trigger
acquire.TriggerAmplitude := 0;    //Trigger at zero volts
acquire.ATriggerAmplitude := 0;
acquire.BTriggerAmplitude := 0;
acquire.TriggerFilter := 0;    //No trigger filter
acquire.TrigSlope := 0;    //rising
acquire.TriggerHoldoff := 0;
acquire.DigPatternRqd := 0;    //not used
acquire.DigPattern :=  0;    //not used
acquire.ExtTrigThreshold := 0;
acquire.DigInputThreshold := 2;
acquire.StartTime :=  -0.005;  //-5 msecs
acquire.StopTime :=   0.005;    //5 msecs
acquire.PreTrigTime := 0.005;
acquire.Port := 0;
acquire.NumDivisions := 10;
acquire.NumSeqFrames := 1;
acquire.NumBuffers := 2;
acquire.SigGenFreq := 1000;    //1kHz output
acquire.SigGenAmp := 2;    //2V amplitude
acquire.SigGenOffset := 0;
acquire.SigGenWaveform := 0;    //sine
acquire.SigGenSweep := 0;
acquire.SigGenFunc := 0;
acquire.SigGenFreq2 := 0;
acquire.SigGenPhase := 0;
acquire.Trig2Function := 0;    //not used
acquire.MinTriggerPeriod := 0.0000001;
acquire.MaxTriggerPeriod := 1;
acquire.TriggerCount := 1;
acquire.Trig2Slope := 0;
acquire.Trig2SourceChan := 0;
acquire.Trig2Level := 0;
acquire.DigPattern2Rqd := 0;
acquire.DigPattern2 := 0;
acquire.Trigger2Source := 0;
acquire.WaveformAverages := 1;
acquire.ValueChanged := 1;
acquire.SamplerResolution := 0; //0 := 10 bit (1 := 12 bit, 2 := 14 bit).
acquire.TransferSize :=0;   //0= normal
samples_required:=1000;
frame_number:=0;
call_control_driver(c_init);
  //initialize the cleverscope acquisition unit
error_code:=  errorOut.code;
end;

//**************************************************
procedure scope_close(var error_code:integer);
{closes cleverscope dll. Returns 0 if all ok, otherwise error code}
begin
call_control_driver(c_finish);
error_code:=  errorOut.code;
end;

//*******************************************************
procedure update_values(a_div, b_div, t_div :double ; number_of_points:integer;
         freq,sigvolts,trigvolts:double; trig_chan:integer; trigger_action:word);
begin
acquire.AMaxScale := v_divisions * a_div / 2;
acquire.AMinScale := - acquire.AMaxScale;
acquire.BMaxScale := v_divisions * b_div / 2;
acquire.BMinScale := - acquire.BMaxScale;
acquire.StopTime := t_divisions * t_div /2;
acquire.StartTime := - acquire.StopTime;
acquire.SigGenFreq := freq;
acquire.SigGenAmp := sigvolts;
acquire.TriggerAmplitude := trigvolts;
acquire.TriggerSource := trig_chan;
samples_required := number_of_points;
acquire.AcquireMode := trigger_action;
acquire.ValueChanged:=acquire.ValueChanged+1;
end;

{---------------------------------------------------------}

procedure scope_config (var error_code:integer);
{configures the acquisition unit. Trig_chan = 0 for Chan A, 1 = chan B
only need to do this if not acquired}

begin

call_control_driver(c_update) ;   //do an update
error_code:=  errorOut.code;
end;

//*************************************************************************

procedure	scope_acquire (var error_code:integer);
{starts an acquisition. returns error code, = 0 if ok}

begin
call_control_driver(c_acquire);	   //Do an acquire
error_code:=  errorOut.code;
end;

//**************************************************************************
procedure scope_read_waveform (var a_waveform, b_waveform : signal;
                 var num_samples:integer; var delta_t, t_zero : double;
                 var error_code:integer);
{reads the samples from the buffer, including waveform details}

var	i: integer;

begin
for i:= 0 to samples_returned do
	begin
   a_waveform[i] := a_samples[i];
	b_waveform[i] := b_samples[i];
	end;

delta_t := dt;
t_zero := t0;
num_samples := samples_returned;
error_code:=  errorOut.code;
end;

//**************************************************************************

function check_for_samples:boolean;
         {returns true if we have got the samples}
begin
call_control_driver(c_check);   {Do a check for samples  }
check_for_samples:= (got_samples = 1);
end;

//************************************************************************

end.
