using System;
using System.Collections.Generic;
using System.ComponentModel;
using System.Data;
using System.Drawing;
using System.Text;
using System.Windows.Forms;
using ZedGraph;

namespace SimpleScope
{
    public partial class Form1 : Form
    {
        public Form1()
        {
            InitializeComponent();
        }
//************************************************************************
//                                APPLICATION CODE
//
// See the timer1_Tick event at end of this file for the place where all the action is.
//
//************************************************************************
const int num_points =2000;


static double 	a_vert = 1.0, b_vert = 1.0, horz = 0.001;
static	double	sig_gen_volts = 1.0, sig_gen_freq = 1000.0;
static	uint 	trig_channel = 0;
static	double	trig_volts = 0.0;
static	bool	single_acquire = false;
static	bool	auto_acquire = false;
//static	double a_waveform[max_samples], b_waveform[max_samples];

static	bool	waiting_for_trigger = false;	//0 means we have not started an acquire. 1 means we are waiting for the trigger
static	ushort	trigger_action;	//for how to capture, 0 = auto, 1 = triggered, 3 = stop
static	bool	vals_changed;


//****************************************************************************

void Setval(System.Windows.Forms.Label  ctrl, double value, string suffix)
{
    ctrl.Text = cscope.eng(value, 3) + suffix;
}

//***********************************************
void InitializeGUI ()
{
single_acquire = false;
auto_acquire = false;
waiting_for_trigger = false;
vals_changed = true;

Setval(Adiv, a_vert,"V");

Setval(Bdiv, b_vert,"V");
Setval(Tdiv, horz,"s");
Setval(TrigV, trig_volts,"V");
Setval(SigGenF, sig_gen_freq,"Hz");
Setval(SigGenV, sig_gen_volts,"V");


SetA_Vertical.Value = 6;
SetB_Vertical.Value = 6;
SetHorizontal.Value = 9;
zedGraphControl1.GraphPane.YAxis.Scale.Min = -a_vert * cscope.v_divisions / 2;
zedGraphControl1.GraphPane.YAxis.Scale.Max = a_vert * cscope.v_divisions / 2;
zedGraphControl1.GraphPane.YAxis.Scale.MajorStep = a_vert;
zedGraphControl1.GraphPane.Y2Axis.Scale.Min = -b_vert * cscope.v_divisions / 2;
zedGraphControl1.GraphPane.Y2Axis.Scale.Max = b_vert * cscope.v_divisions / 2;
zedGraphControl1.GraphPane.Y2Axis.Scale.MajorStep = b_vert;
zedGraphControl1.GraphPane.XAxis.Scale.Min = -horz * cscope.v_divisions / 2;
zedGraphControl1.GraphPane.XAxis.Scale.Max = horz * cscope.v_divisions / 2;
zedGraphControl1.GraphPane.XAxis.Scale.MajorStep = horz;
Trig_A.Checked = true;
timer1.Enabled = true;
}
//*************************************************************************
// Build the Chart
void CreateGraph( ZedGraphControl zgc )
{
   // get a reference to the GraphPane
   GraphPane myPane = zgc.GraphPane;

   // Set the Titles
   myPane.Title.Text = "Simple Scope";
   myPane.XAxis.Title.Text = "Time (s)";
   myPane.YAxis.Title.Text = "A Volts";
   myPane.Y2Axis.Title.Text = "B Volts";

   // Make up some data arrays based on the Sine function
   double x, y1, y2;
   PointPairList list1 = new PointPairList();
   PointPairList list2 = new PointPairList();
   for ( int i = 0; i < 2; i++ )
   {
      x = 0;
	  y1 = 0;
	  y2 = 0;
      list1.Add( x, y1 );
      list2.Add( x, y2 );
   }

   // Generate a red curve with diamond
   // symbols, and "Porsche" in the legend
   LineItem myCurve = myPane.AddCurve( "",
	   list1, Color.Red, SymbolType.None );

   // Generate a blue curve with circle
   // symbols, and "Piper" in the legend
   myCurve = myPane.AddCurve( "",
	   list2, Color.Blue, SymbolType.None );

	// Associate this curve with the Y2 axis
	myCurve.IsY2Axis = true;

	// Show the x axis grid
	myPane.XAxis.MajorTic.IsOpposite = false;
	myPane.XAxis.MinorTic.IsOpposite = false;
	myPane.XAxis.MajorGrid.IsVisible = true;


	// Make the Y axis scale red
	myPane.YAxis.Scale.FontSpec.FontColor = Color.Red;
	myPane.YAxis.Title.FontSpec.FontColor = Color.Red;
	// turn off the opposite tics so the Y tics don't show up on the Y2 axis
	myPane.YAxis.MajorTic.IsOpposite = false;
	myPane.YAxis.MinorTic.IsOpposite = false;
	// Don't display the Y zero line
	myPane.YAxis.MajorGrid.IsZeroLine = false;
	myPane.YAxis.MajorGrid.IsVisible = true;
	// Align the Y axis labels so they are flush to the axis
	myPane.YAxis.Scale.Align = AlignP.Inside;
	// Manually set the axis range
	myPane.YAxis.Scale.Min = -10;
	myPane.YAxis.Scale.Max = 10;


	// Enable the Y2 axis display
	myPane.Y2Axis.IsVisible = true;
	// Make the Y2 axis scale blue
	myPane.Y2Axis.Scale.FontSpec.FontColor = Color.Blue;
	myPane.Y2Axis.Title.FontSpec.FontColor = Color.Blue;
	// turn off the opposite tics so the Y2 tics don't show up on the Y axis
	myPane.Y2Axis.MajorTic.IsOpposite = false;
	myPane.Y2Axis.MinorTic.IsOpposite = false;
	// Display the Y2 axis grid lines
	myPane.Y2Axis.MajorGrid.IsVisible = false;
	// Align the Y2 axis labels so they are flush to the axis
	myPane.Y2Axis.Scale.Align = AlignP.Inside;
	myPane.Y2Axis.MajorGrid.IsZeroLine = false;



   // Tell ZedGraph to refigure the
   // axes since the data have changed
   zgc.AxisChange();
   zgc.Invalidate();
}

//--------------------------------------------------------------------------


        private void QuitButton_Click(object sender, EventArgs e)
        {
            timer1.Enabled = false;
            cscope.scope_close();
            this.Close();

        }
        //---------------------------------------------------------------------------------------

        private void Form1_Load(object sender, EventArgs e)
        {
            int nErr;

            // Setup the graph
            CreateGraph(zedGraphControl1);
            nErr = 0;
            nErr = cscope.scope_init();
            if (nErr > 0)
            {
            }
            else
            {
                InitializeGUI();
            }
        }
        //---------------------------------------------------------------------------------------

        private void SetA_Vertical_ValueChanged(object sender, EventArgs e)
        {
        a_vert = cscope.v_div[SetA_Vertical.Value];
        Setval(Adiv, a_vert, "V");
        vals_changed = true;
        zedGraphControl1.GraphPane.YAxis.Scale.Min = -a_vert * cscope.v_divisions / 2;
        zedGraphControl1.GraphPane.YAxis.Scale.Max = a_vert * cscope.v_divisions / 2;
        zedGraphControl1.GraphPane.YAxis.Scale.MajorStep = a_vert;
        zedGraphControl1.GraphPane.AxisChange();
        zedGraphControl1.Invalidate();
        }
//---------------------------------------------------------------------------------------

        private void SetB_Vertical_ValueChanged(object sender, EventArgs e)
        {
            b_vert = cscope.v_div[SetB_Vertical.Value];
        Setval(Bdiv, b_vert, "V");
        vals_changed = true;
        zedGraphControl1.GraphPane.Y2Axis.Scale.Min = -b_vert * cscope.v_divisions / 2;
        zedGraphControl1.GraphPane.Y2Axis.Scale.Max = b_vert * cscope.v_divisions / 2;
        zedGraphControl1.GraphPane.Y2Axis.Scale.MajorStep = b_vert;
        zedGraphControl1.GraphPane.AxisChange();
        zedGraphControl1.Invalidate();
        }
        //---------------------------------------------------------------------------------------

        private void SetHorizontal_ValueChanged(object sender, EventArgs e)
        {
            horz = cscope.t_div[SetHorizontal.Value];
        Setval(Tdiv, horz, "sec");
        vals_changed = true;
        zedGraphControl1.GraphPane.XAxis.Scale.Min = -horz * cscope.t_divisions / 2;
        zedGraphControl1.GraphPane.XAxis.Scale.Max = horz * cscope.t_divisions / 2;
        zedGraphControl1.GraphPane.XAxis.Scale.MajorStep = horz;
        zedGraphControl1.GraphPane.AxisChange();
        zedGraphControl1.Invalidate();
        }
        //---------------------------------------------------------------------------------------

        private void Set_SigGen_Freq_ValueChanged(object sender, EventArgs e)
        {
            sig_gen_freq = cscope.freqs[Set_SigGen_Freq.Value];
            Setval(SigGenF, sig_gen_freq, "Hz");
            vals_changed = true;
        }
        //---------------------------------------------------------------------------------------

        private void Set_SigGen_Volts_ValueChanged(object sender, EventArgs e)
        {
            sig_gen_volts = Set_SigGen_Volts. Value * 0.2;
            Setval(SigGenV, sig_gen_volts, "V");
            vals_changed = true;

        }
        //---------------------------------------------------------------------------------------

        private void Set_Trigger_Volts_ValueChanged(object sender, EventArgs e)
        {
            trig_volts = Set_Trigger_Volts. Value * 0.2;
            Setval(TrigV, trig_volts, "V");
            vals_changed = true;

        }
        //---------------------------------------------------------------------------------------

        private void Trig_A_CheckedChanged(object sender, EventArgs e)
        {
            trig_channel = 0;
            vals_changed = true;

        }
        //---------------------------------------------------------------------------------------

        private void Trig_B_CheckedChanged(object sender, EventArgs e)
        {
            trig_channel = 1;
            vals_changed = true;

        }
        //---------------------------------------------------------------------------------------

    private void single_trigger_Click(object sender, EventArgs e)
    	{
    	vals_changed = true;
			Trig_LED.Visible = false;
			auto_acquire = false;
			waiting_for_trigger = false;
			TriggeredAcquire.Text="Auto";
			if (single_acquire)
				{
				single_acquire = false;
				single_trigger.Text="Single";
        trigger_action = cscope.acq_stop;

            //stop doing stuff
				}
			else
				{
				single_acquire = true;
				single_trigger.Text="Wait..";
      	      trigger_action = cscope.acq_single;   //use triggered captured
				}
      }
        //---------------------------------------------------------------------------------------

        private void TriggeredAcquire_Click(object sender, EventArgs e)
        {
		vals_changed = true;
		single_acquire = false;
		Trig_LED.Visible = false;
		waiting_for_trigger = false;
		single_trigger.Text="Single";
		if (auto_acquire)
			{
			auto_acquire = false;
			TriggeredAcquire.Text="Auto";
            trigger_action = cscope.acq_stop;   //stop doing stuff
			}
		else
			{
			auto_acquire = true;
			TriggeredAcquire.Text="Triggering...";
            trigger_action = cscope.acq_auto;   //use auto captured
			}
        }
        //---------------------------------------------------------------------------------------

        private void timer1_Tick(object sender, EventArgs e)
        {
            int i;
            double xval;
            uint num_samples = 0;
            double dT = 0;
            double T0 = 0;
            float[] a_waveform = new float[cscope.max_samples];
            float[] b_waveform = new float[cscope.max_samples];


            if (vals_changed)	//here we put the latest set of updates in the acquisition array
                                //these will be actioned when we do an acquire, or down below if there is no acquire.
                {
                 cscope.update_values(a_vert, b_vert, horz, num_points,
                 sig_gen_freq, sig_gen_volts, trig_volts, trig_channel, trigger_action);
                }
            if ((!waiting_for_trigger) && (single_acquire || auto_acquire))
                {
                Trig_LED.Visible = false;
                waiting_for_trigger = true;
                vals_changed = false;		//will have been updated by acquire. We do it here so that if vals_changed
                cscope.scope_acquire();  //is set true while acquiring, there
                }
            if (waiting_for_trigger && cscope.check_for_samples())
                {
                Trig_LED.Visible = true;
                waiting_for_trigger = false;
                if (single_acquire) single_trigger.Text = "Single";
                single_acquire = false;
                cscope.scope_read_waveform(a_waveform, b_waveform, ref num_samples, ref dT, ref T0);
                xval = T0;
                IPointListEdit ip1 = zedGraphControl1.GraphPane.CurveList[0].Points as IPointListEdit;  //           CurveList[L"Beta"].Points;
                IPointListEdit ip2 = zedGraphControl1.GraphPane.CurveList[1].Points as IPointListEdit;  //           CurveList[L"Beta"].Points;

                if ((ip1 != null) && (ip2 != null))
                    {
                    ip1.Clear();
                    ip2.Clear();
                    for (i = 0; i < num_samples; i++)
                        {
                        ip1.Add(xval, a_waveform[i]);
                        ip2.Add(xval, b_waveform[i]);
                        xval = xval + dT;
                        }
                    }
                zedGraphControl1.Invalidate();
                }
                if (vals_changed)
                    {					//vals have changed that couldn't be handled by the acquire.
                    cscope.update_values(a_vert, b_vert, horz, num_points,
                    sig_gen_freq, sig_gen_volts, trig_volts, trig_channel, trigger_action);
                    cscope.scope_config();
                    vals_changed = false;		//updated if acquire did not update them
                    }

            }

//---------------------------------------------------------------------------------------

        }
}
