using System;
using System.Runtime.InteropServices;

class	cscope
{

const	ushort		c_init		= 0;
const	ushort		c_acquire	= 1;
const	ushort		c_replay	= 2;
const	ushort		c_check		= 3;
const	ushort		c_update	= 4;
const	ushort		c_finish	= 5;

public const	uint	max_samples=16384;
public const ushort t_divisions = 10;
public const ushort v_divisions = 8;

//Trigger actions:
public const ushort acq_single = 0;
public const ushort acq_auto = 1;
public const ushort acq_stop = 3;

public  static double [] v_div = { 0.01, 0.02, 0.05, 0.1, 0.2, 0.5, 1, 2, 5};
public static double[] t_div = { 1,0.5,0.2,0.1,0.05,0.02,0.01,0.005,0.002,0.001,
					  500e-6,200e-6,100e-6, 50e-6,20e-6,10e-6,5e-6,2e-6,1e-6,
					  500e-9,200e-9,100e-9,50e-9,20e-9,10e-9};
public static double[] freqs = {1.0,2,5,10,20,50,100,200,500,1000,2000,5000,10000,
					 20000,50000,100000,200000,500000,1000000,2000000,5000000};

static	float	[]	a_samples = new float[max_samples];	  	//contains the a channel samples
static	float	[]	b_samples = new float[max_samples];	  	//contains the b channel samples
static	ushort	[]	dig_samples = new ushort[max_samples];	//contains teh digotal channel values


static	int		samples_required;			//the number of samples to capture and display
public static	uint		samples_returned;			//actual number of samples returned
public static	double		dt,t0;						//holds time increment and start value
static	uint		frames_returned;			//number of frames returned after sequential transfer
public static	CSCOPE_ACQUIRE	acquire;				//holds the acquire definition
static	CSCOPE_ERROR    scope_error;					//holds the error value
static	byte	got_samples;	//set to 1 when we have samples
static	int 		frame_number;				//frame number to extract for multi-frame replay

//***************************************************

static int call_cscope_control_driver(ushort command)
//use this routine to call the control driver with a particular command
{
    Cscope_driver.CscopeControlDriver(command, acquire.StartTime, acquire.StopTime, samples_required, frame_number, ref acquire, ref got_samples,
                    ref t0, ref dt, ref samples_returned, ref frames_returned, a_samples, max_samples * sizeof(double), b_samples, max_samples * sizeof(double),
                    dig_samples, max_samples * sizeof(ushort), ref scope_error);
    return scope_error.err_code;
}

//**************************************************************
public	static int	scope_init()
//Assumes the scope is connected and opens it.
//Sets up the default values

{
acquire.AcquireMode = 1;		//automatic  (0 = single, 1 = automatic, 2 = triggered, 3 = stop)
acquire.AcquisitionMode = 1;	//peak captured
acquire.Acquirer =  4;			//cleverscope is the acquirer
acquire.TransferChans = 2;		//transfer both channels
acquire.AMaxScale = 2;			// Volts range = +/-2
acquire.AMinScale = -2;
acquire.BMaxScale = 2;
acquire.BMinScale = -2;
acquire.AProbe = 0;				//x1
acquire.BProbe = 0;				//x1
acquire.ACoupling = 1;			//DC
acquire.BCoupling = 1;			//DC
acquire.ABandwidth = 1;			//100 MHz
acquire.BBandwidth = 1;			//100 MHz
acquire.TriggerSource = 0;		//A Chan trigger
acquire.TriggerAmplitude = 0;	//Trigger at zero volts
acquire.ATriggerAmplitude = 0;
acquire.BTriggerAmplitude = 0;
acquire.TriggerFilter = 0;		//No trigger filter
acquire.TrigSlope = 0;			//rising
acquire.TriggerHoldoff = 0;
acquire.DigPatternRqd = 0;		//not used
acquire.DigPattern =  0;		//not used
acquire.ExtTrigThreshold = 0;
acquire.DigInputThreshold = 2;
acquire.StartTime =  -0.005;	//-5 msecs
acquire.StopTime =   0.005;		//5 msecs
acquire.PreTrigTime = 0.005;
acquire.Port = 0;
acquire.NumDivisions = 10;
acquire.NumSeqFrames = 1;
acquire.NumBuffers = 2;
acquire.SigGenFreq = 1000;		//1kHz output
acquire.SigGenAmp = 1;			//1V amplitude
acquire.SigGenOffset = 0;
acquire.SigGenWaveform = 0;		//sine
acquire.SigGenSweep = 0;
acquire.SigGenFunc = 0;
acquire.SigGenFreq2 = 0;
acquire.SigGenPhase = 0;
acquire.Trig2Function = 0;		//not used
acquire.MinTriggerPeriod = 0.0000001;
acquire.MaxTriggerPeriod = 1;
acquire.TriggerCount = 1;
acquire.Trig2Slope = 0;
acquire.Trig2SourceChan = 0;
acquire.Trig2Level = 0;
acquire.DigPattern2Rqd = 0;
acquire.DigPattern2 = 0;
acquire.Trigger2Source = 0;
acquire.WaveformAverages = 1;
acquire.ValueChanged = 1;
acquire.SamplerResolution = 0;	//0 = 10 bit (1 = 12 bit, 2 = 14 bit).
samples_required = 1000;
frame_number = 0;

return call_cscope_control_driver(c_init);
}



//*************************************************
public static int scope_close()
//closes the scope
{
	return call_cscope_control_driver(c_finish);
}


//****************************************************
public static void update_values(double a_div, double b_div, double time_div, int number_of_points,
					double freq, double sigvolts, double trigvolts, uint trig_chan, ushort trigger_action)
//updates the acquire variable only. If triggered = 1 then we do triggered capture, otherwise auto capture
{
acquire.AMaxScale = v_divisions * a_div / 2;
acquire.AMinScale = - acquire.AMaxScale;
acquire.BMaxScale = v_divisions * b_div / 2;
acquire.BMinScale = - acquire.BMaxScale;
acquire.StopTime = t_divisions * time_div /2;
acquire.StartTime = - acquire.StopTime;
acquire.SigGenFreq = freq;
acquire.SigGenAmp = sigvolts;
acquire.TriggerAmplitude = trigvolts;
acquire.TriggerSource = trig_chan;
samples_required = number_of_points;
acquire.AcquireMode = trigger_action; //0= auto, 1 = triggered, 3 = stop
acquire.ValueChanged++;
}

//*******************************************************
public static int scope_config()
//Configures major values for the acquisition unit.
{
return call_cscope_control_driver(c_update);
}

//*************************************************************************

    public static int scope_acquire()
//start an acquisition

{
return call_cscope_control_driver(c_acquire);
}

//**************************************************************************
    public static int scope_read_waveform(float[] a_waveform, float[] b_waveform, ref uint num_samples, ref double delta_t, ref double t_zero)
//returns the last read waveform for the given channel
{
uint i;

for (i=0; i<samples_returned; i++)
	{
	a_waveform[i] = a_samples[i];
	}
for (i=0; i<samples_returned; i++)
	{
	b_waveform[i] = b_samples[i];
	}

delta_t = dt;
t_zero = t0;
num_samples = samples_returned;
return scope_error.err_code;
}

//**************************************************************************

    public static bool check_for_samples()
//checks to see if samples have been returned. If so returns 1, else 0
{
call_cscope_control_driver(c_check);
return (got_samples !=0);
}

//****************************************************************************

//Support routines



//-------------------------------------------------------------------------
/*
    public static string eng(double value, int digits)
{
return String.Format("{0:G"+digits.ToString()+"}", value);
}
*/
//------------------------------------------------------------------------
//-------------------------------------------------------------------------
const	short PREFIX_START= -24;
const	short PREFIX_END= 24;
/* Smallest power of then for which there is a prefix defined.
   If the set of prefixes will be extended, change this constant
   and update the table "prefix". */

    public static string eng(double value, int digits)
{
  string prefix = "yzafpnum kMGTPEZY";


      int expof10;
      bool negative = false;
      string estr = "                   ";
      if (value < 0.0)
        {
            value = -value;
           negative= true;
        }
		if (value != 0.0)
			{
			expof10 = (int) Math.Log10(value);
		   if(expof10 > 0)
			 expof10 = (expof10/3)*3;
		   else
			 expof10 = (-expof10+3)/3*(-3);

		   value *= Math.Pow(10.0,-expof10);

		   if (value >= 1000.0)
			  { value /= 1000.0; expof10 += 3; }
		   else if(value >= 100.0)
			  digits -= 2;
		   else if(value >= 10.0)
			  digits -= 1;
		   }
	   else
			{
			expof10 = 0;	//no prefix
			digits=3;
			}
      if((expof10 < PREFIX_START) || (expof10 > PREFIX_END))
      {}

 //       sprintf_s(res,20, "%.*fe%d", digits-1, value, expof10); 
      else
          estr= string.Format("{0:f"+(digits-1).ToString()+"}", value)+prefix[(expof10-PREFIX_START)/3];
//        sprintf_s(res,20, "%.*f %s", digits-1, value, 
 //         prefix[(expof10-PREFIX_START)/3]);
      if (negative) estr = "-" + estr;
      return estr;
}

//------------------------------------------------------------------------




} //class


