#include "Cscope Control Driver.h"
#include "cscope interface.h"
#include <stdlib.h>
#include <stdio.h>
#include <math.h>

#define	c_init		0
#define	c_acquire	1
#define	c_replay	2
#define	c_check		3
#define	c_update	4
#define	c_finish	5



int32 scope_err;
 


int32		samples_required;			//the number of samples to capture and display
uInt32		samples_returned;			//actual number of samples returned
double		dt,t0;						//holds time increment and start value
uInt32		frames_returned;			//number of frames returned after sequential transfer
static	   	TD1	acquire;				//holds the acquire definition
static		TD2	error;					//holds the error value
static		LVBoolean	got_samples;	//set to 1 when we have samples
int32 		frame_number;				//frame number to extract for multi-frame replay

//**************************************************

int32 call_cscope_control_driver(uInt16 command)
//use this routine to call the control driver with a particular command
{
CscopeControlDriver(command, acquire.StartTime, acquire.StopTime, samples_required,frame_number, &acquire, &got_samples,
                    &t0, &dt, &samples_returned, &frames_returned, a_samples, sizeof(a_samples),  b_samples, sizeof(b_samples),
                    dig_samples, sizeof(dig_samples), &error);
                    
scope_err = error.code;                    
return scope_err;
}

//**************************************************************
int32	scope_init (void)
//Assumes the scope is connected and opens it. 
//Sets up the default values

{
acquire.AcquireMode = 1;		//automatic  (0 = single, 1 = automatic, 2 = triggered, 3 = stop)
acquire.AcquisitionMode = 1;	//peak captured
acquire.Acquirer =  4;			//cleverscope is the acquirer
acquire.TransferChans = 2;		//transfer both channels
acquire.AMaxScale = 2;			// Volts range = +/-2
acquire.AMinScale = -2;
acquire.BMaxScale = 2;
acquire.BMinScale = -2;
acquire.AProbe = 0;				//x1
acquire.BProbe = 0;				//x1
acquire.ACoupling = 1;			//DC
acquire.BCoupling = 1;			//DC
acquire.ABandwidth = 1;			//100 MHz
acquire.BBandwidth = 1;			//100 MHz
acquire.TriggerSource = 0;		//A Chan trigger
acquire.TriggerAmplitude = 0;	//Trigger at zero volts
acquire.ATriggerAmplitude = 0;
acquire.BTriggerAmplitude = 0;
acquire.TriggerFilter = 0;		//No trigger filter
acquire.TrigSlope = 0;			//rising 
acquire.TriggerHoldoff = 0;
acquire.DigPatternRqd = 0;		//not used 
acquire.DigPattern =  0;		//not used
acquire.ExtTrigThreshold = 0;
acquire.DigInputThreshold = 2;
acquire.StartTime =  -0.005;	//-5 msecs
acquire.StopTime =   0.005;		//5 msecs
acquire.PreTrigTime = 0.005;
acquire.Port = 0;
acquire.NumDivisions = 10;
acquire.NumSeqFrames = 1; 
acquire.NumBuffers = 2; 
acquire.SigGenFreq = 1000;		//1kHz output
acquire.SigGenAmp = 1;			//1V amplitude
acquire.SigGenOffset = 0;
acquire.SigGenWaveform = 0;		//sine
acquire.SigGenSweep = 0;
acquire.SigGenFunc = 0; 
acquire.SigGenFreq2 = 0;
acquire.SigGenPhase = 0;
acquire.Trig2Function = 0;		//not used
acquire.MinTriggerPeriod = 0.0000001;
acquire.MaxTriggerPeriod = 1; 
acquire.TriggerCount = 1;
acquire.Trig2Slope = 0;
acquire.Trig2SourceChan = 0;
acquire.Trig2Level = 0;
acquire.DigPattern2Rqd = 0;
acquire.DigPattern2 = 0;
acquire.Trigger2Source = 0;
acquire.WaveformAverages = 1;
acquire.ValueChanged = 1;
acquire.SamplerResolution = 0;	//0 = 10 bit (1 = 12 bit, 2 = 14 bit). 
samples_required = 1000;
frame_number = 0;

return call_cscope_control_driver(c_init); 
}



//*************************************************
int32 scope_close (void)
//closes the scope
{
	int32	err;
	err = call_cscope_control_driver(c_finish);
	return (err);
}


//****************************************************
void update_values(double a_div, double b_div, double t_div, int32 number_of_points, 
					double freq, double sigvolts, double trigvolts, int32 trig_chan, uInt16 trigger_action)
//updates the acquire variable only. If triggered = 1 then we do triggered capture, otherwise auto capture
{
acquire.AMaxScale = v_divisions * a_div / 2;
acquire.AMinScale = - acquire.AMaxScale;
acquire.BMaxScale = v_divisions * b_div / 2;
acquire.BMinScale = - acquire.BMaxScale;
acquire.StopTime = t_divisions * t_div /2;
acquire.StartTime = - acquire.StopTime;  
acquire.SigGenFreq = freq;
acquire.SigGenAmp = sigvolts;
acquire.TriggerAmplitude = trigvolts;
acquire.TriggerSource = trig_chan;	
samples_required = number_of_points; 
acquire.AcquireMode = trigger_action; //0= auto, 1 = triggered, 3 = stop
acquire.ValueChanged++;
}

//*******************************************************
int32 scope_config (void)
//Configures major values for the acquisition unit. 
{
return call_cscope_control_driver(c_update);  
}

//*************************************************************************

int32	scope_acquire (void)
//start an acquisition

{
return call_cscope_control_driver(c_acquire); 
}

//**************************************************************************
int32 scope_read_waveform (float a_waveform[max_samples], float b_waveform[max_samples], int32 *num_samples, double *delta_t, double *t_zero) 
//returns the last read waveform for the given channel
{
uInt32 i;

for (i=0; i<samples_returned; i++)
	{
	a_waveform[i] = a_samples[i];
	}
for (i=0; i<samples_returned; i++)
	{
	b_waveform[i] = b_samples[i];
	}

*delta_t = dt;
*t_zero = t0;
*num_samples = samples_returned;
return scope_err;
}

//**************************************************************************

int32 check_for_samples(void)
//checks to see if samples have been returned. If so returns 1, else 0
{
call_cscope_control_driver(c_check);
return got_samples;
}

//****************************************************************************

//Support routines



//-------------------------------------------------------------------------
#define PREFIX_START (-24)
/* Smallest power of then for which there is a prefix defined.
   If the set of prefixes will be extended, change this constant
   and update the table "prefix". */

char *eng(double value, int digits, int numeric)
{
  static char *prefix[] = {
  "y", "z", "a", "f", "p", "n", "u", "m", "",
  "k", "M", "G", "T", "P", "E", "Z", "Y"
  }; 
#define PREFIX_END (PREFIX_START+\
(int)((sizeof(prefix)/sizeof(char *)-1)*3))

      int expof10;
      static char result[100];
      char *res = result;

      if (value < 0.)
        {
            *res++ = '-';
            value = -value;
        }
		if (value != 0.0)
			{
			expof10 = (int) log10(value);
		   if(expof10 > 0)
			 expof10 = (expof10/3)*3;
		   else
			 expof10 = (-expof10+3)/3*(-3);

		   value *= pow(10.0,-expof10);

		   if (value >= 1000.)
			  { value /= 1000.0; expof10 += 3; }
		   else if(value >= 100.0)
			  digits -= 2;
		   else if(value >= 10.0)
			  digits -= 1;
		   }
	   else
			{
			expof10 = 0;	//no prefix
			digits=3;
			}
      if(numeric || (expof10 < PREFIX_START) ||    
                    (expof10 > PREFIX_END))
        sprintf_s(res,20, "%.*fe%d", digits-1, value, expof10); 
      else
        sprintf_s(res,20, "%.*f %s", digits-1, value, 
          prefix[(expof10-PREFIX_START)/3]);
      return result;
}

//------------------------------------------------------------------------




